package gz.cyberbrain.cyberhotel.model.owe.dao;

import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.spring.dao.AbstractSpringDao;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;

public class OweSpringDao extends AbstractSpringDao implements OweDao {

	public OweSpringDao() {
		super(OweTo.class);
	}	

	public OweTo create(OweTo oweTo) throws InternalErrorException {		

		return (OweTo) super.save(oweTo);

	}

	public OweTo find(Long oweNumber) throws InstanceNotFoundException, InternalErrorException {

		String queryName = "anOwe"; 
		Collection oweTo = new ArrayList();
		oweTo = super.findByAttribute(queryName,oweNumber);
		Exception e = new Exception("Owe not unique!");
		if (oweTo.size() > 1) throw new InternalErrorException(e);
		if (oweTo.isEmpty()) throw new InstanceNotFoundException(oweNumber,OweTo.class.getName());
		Iterator iter = oweTo.iterator();
		
		return (OweTo) iter.next();

	}

	public void update(OweTo oweTo) throws InternalErrorException {		

		super.update(oweTo);

	}

	public void remove(OweTo oweTo) throws InternalErrorException {

			super.delete(oweTo);
	
	}
	
	public Collection findAll() throws InternalErrorException {
		
		String queryName = "allOwes";
		return super.findByQuery(queryName);
		
	}
	
	public Collection findByRoom(String roomCode) throws InternalErrorException {
		String queryName = "owesByRoom";
		return super.findByAttribute(queryName, roomCode);
	}

	public Collection findByParticular(Long particularId) throws InternalErrorException {
		String queryName = "owesByParticular";
		return super.findByAttribute(queryName, particularId);
	}
	
	public Collection findByRoomAndParticular (String roomCode, Long particularId) 
	throws InternalErrorException {
		
		String queryName = "owesByRoomAndParticular";
		Serializable []attributes = {roomCode, particularId};
		return super.findByAttributes(queryName, attributes);
		
	}

	public Collection findByReservation (Long reservationId) throws InternalErrorException {
		
		String queryName = "owesByReservation";
		return super.findByAttribute(queryName, reservationId);
		
	}
	
	public Long nextOwe(Long reservationId) throws InternalErrorException {
		
		String queryName = "nextOwe";
		Exception e = new Exception("Owe id generator fails!");
		Collection oweNumber = new ArrayList();
		oweNumber = super.findByAttribute(queryName, reservationId);
		if (oweNumber.size() > 1) throw new InternalErrorException(e);
		Iterator iter = oweNumber.iterator();
		Integer next = ((Integer) iter.next());
		
		return (next != null? next.longValue() : null);
		
	}

	public Collection findByDate(Calendar date) throws InternalErrorException {
		String queryName = "owesByDate";	
		return super.findByAttribute(queryName, date);
	}

	public Long findOweReservation(Calendar date, String roomCode) throws InstanceNotFoundException, InternalErrorException{
		
		String queryName = "oweReservation";
		Serializable []attributes = {date, date, roomCode};
		Collection reservationTo = new ArrayList();
		reservationTo = super.findByAttributes(queryName, attributes);
		Exception e = new Exception("Reservation not unique!");
		if (reservationTo.size() > 1) throw new InternalErrorException(e);
		if (reservationTo.isEmpty()) throw new InstanceNotFoundException(roomCode + "-" + date, ReservationTo.class.getName());
		Iterator iter = reservationTo.iterator();
		
		return (Long) iter.next();
		
	}
	
}
