package gz.cyberbrain.cyberhotel.view.swing.edit;

import gz.cyberbrain.cyberhotel.controller.SeasonController;
import gz.cyberbrain.cyberhotel.controller.SeasonListAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.LongItem;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.util.Calendar;
import java.util.Date;
import java.util.SortedSet;
import java.util.Vector;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JInternalFrame;
import javax.swing.JPopupMenu;

import org.jdesktop.swingx.calendar.JXMonthView;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Toolkit;

import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.border.LineBorder;

public class CalendarWindow extends JInternalFrame {

	private static final long serialVersionUID = 1L;
	private static CalendarWindow calendarWindow = null;
	private JPanel jContentPane = null;
	private JXMonthView calendar = null;
	private JPopupMenu seasonsMenu = null;
	private InfoPanel infoPanel = null;
	private JPanel firstPanel = null;
	private JPanel seasonsPanel = null;
	private JLabel seasonsLabel = null;
	private JComboBox seasonsComboBox = null;
	private JButton addButton = null;
	private SeasonListAdapter adapter = null;
	private JButton delButton = null;

	/**
	 * This is the xxx default constructor
	 */
	private CalendarWindow() {
		super("", false, true, false, true);
		initialize();
	}

	public static CalendarWindow getInstance() {
		if (calendarWindow == null) calendarWindow = new CalendarWindow();
		return calendarWindow;
	}	

	public void setAdapter(SeasonListAdapter adapter) {
		this.adapter = adapter;
	}

	public SeasonListAdapter getAdapter() {
		return adapter;
	}

	public InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(0,0,this.getWidth(),50);
			infoPanel.setInfoText(Locale.get("CalendarWindow.infoText"));
		}
		return infoPanel;
	}

	private JPanel getFirstPanel() {
		if (firstPanel == null) {
			firstPanel = new JPanel();
			firstPanel.setBounds(0, 50, this.getWidth(), this.getHeight() - 145);
			firstPanel.add(getCalendar());
			firstPanel.add(getSeasonsMenu());
		}
		return firstPanel;
	}

	public void setInfoPanel(InfoPanel infoPanel) {
		this.infoPanel = infoPanel;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(925, 682);

		this.setResizable(false);

		this.setContentPane(getJContentPane());
		this.setAdapter(new SeasonListAdapter());
		selectDays();
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
		{   

			public void internalFrameClosing(javax.swing.event.InternalFrameEvent evt)
			{        
				calendarWindow = null;				
			}
		});	
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getInfoPanel());
			jContentPane.add(getFirstPanel());
			jContentPane.add(getSeasonsPanel(), null);
		}
		return jContentPane;
	}

	public JXMonthView getCalendar() {
		if (calendar == null) {
			calendar = new JXMonthView();
			calendar.setBounds(0, 0, this.getWidth(), this.getHeight() - 150);
			calendar.setPreferredCols(4);
			calendar.setPreferredRows(3);
			calendar.setFirstDayOfWeek(Calendar.MONDAY);
			calendar.setSelectionMode(JXMonthView.SelectionMode.MULTIPLE_INTERVAL_SELECTION);
			calendar.setTraversable(true);
			calendar.setSelectedBackground(Color.PINK);
		}
		return calendar;
	}

	public void setCalendar(JXMonthView calendar) {
		this.calendar = calendar;
	}

	private JPopupMenu getSeasonsMenu() {
		if (seasonsMenu == null) {

			seasonsMenu = new JPopupMenu();

			Vector seasons = SeasonController.seasonList();
			for (int i=0; i<seasons.size(); i++) {

				JMenuItem item = new JMenuItem(((TextItem) seasons.elementAt(i)).getText());
				seasonsMenu.add(item);

			}


		}
		return seasonsMenu;
	}

	/**
	 * This method initializes seasonsPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getSeasonsPanel() {
		if (seasonsPanel == null) {
			seasonsLabel = new JLabel();
			seasonsLabel.setBounds(new Rectangle(8, 7, 67, 16));
			seasonsLabel.setText(Locale.get("CalendarWindow.seasonsLabel"));
			seasonsPanel = new JPanel();
			seasonsPanel.setLayout(null);
			seasonsPanel.setBounds(new Rectangle(10, 593, 317, 50));
			seasonsPanel.setBorder(new LineBorder(Color.BLACK));
			seasonsPanel.add(seasonsLabel, null);
			seasonsPanel.add(getSeasonsComboBox(), null);
			seasonsPanel.add(getAddButton(), null);
			seasonsPanel.add(getDelButton(), null);
		}
		return seasonsPanel;
	}

	/**
	 * This method initializes seasonsComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getSeasonsComboBox() {
		if (seasonsComboBox == null) {

			seasonsComboBox = new JComboBox(SeasonController.seasonList());
			seasonsComboBox.setBounds(new Rectangle(85, 9, 136, 17));
			seasonsComboBox.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					unselectDays();
					selectDays();
				}
			});
		}
		return seasonsComboBox;
	}

	/**
	 * This method initializes seasonsButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton();
			addButton.setBounds(new Rectangle(234, 6, 74, 17));
			addButton.setText(Locale.get("CalendarWindow.addButton"));
			addButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					SortedSet dates = getCalendar().getSelection();
					System.out.println("dates size = " + dates.size());
					getAdapter().updateDays(dates, (Long) ((TextItem) getSeasonsComboBox().getSelectedItem()).getId());

					Object[] options = {Locale.get("JOptionPane.Ok")};
					JOptionPane.showOptionDialog(CalendarWindow.getInstance(),
							Locale.get("JOptionPane.CalendarWindow.addButton.message"),
							Locale.get("JOptionPane.CalendarWindow.addButton.title"),
							JOptionPane.OK_OPTION,
							JOptionPane.INFORMATION_MESSAGE,
							null,
							options,
							options[0]);
				}

			});

		}
		return addButton;
	}

	private void selectDays() {		

		Vector days = getAdapter().seasonDays((Long) ((TextItem) seasonsComboBox.getSelectedItem()).getId());

		for (int i=0; i<days.size(); i++) {
			Date day = ((Calendar) ((LongItem) days.elementAt(i)).getId()).getTime();			
			getCalendar().addSelectionInterval(day, day);
		}		

		getCalendar().repaint();

	}

	private void unselectDays() {

		getCalendar().removeSelectionInterval(getCalendar().getSelection().first(), getCalendar().getSelection().last());
		getCalendar().repaint();

	}

	/**
	 * This method initializes delButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getDelButton() {
		if (delButton == null) {
			delButton = new JButton();
			delButton.setBounds(new Rectangle(234, 28, 74, 17));
			delButton.setText(Locale.get("CalendarWindow.delButton"));
			delButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					Object[] options = {Locale.get("JOptionPane.Yes"), Locale.get("JOptionPane.No")};
					int n = JOptionPane.showOptionDialog(CalendarWindow.getInstance(),
							Locale.get("JOptionPane.CalendarWindow.delButton.message"),
							Locale.get("JOptionPane.CalendarWindow.delButton.title"),
							JOptionPane.YES_NO_OPTION,
							JOptionPane.INFORMATION_MESSAGE,
							null,
							options,
							options[0]);

					if (n == JOptionPane.YES_OPTION) {												

						SortedSet dates = getCalendar().getSelection();
						getAdapter().removeSeasons(dates);

						if (!inputErrors()) {
							unselectDays();
							selectDays();
						}

					}

				}

			});
		}
		return delButton;
	}

	private boolean inputErrors() {

		if (!ErrorsHandler.getErrors().isEmpty()) {

			if (ErrorsHandler.getErrorAt(0).getItem().equals("seasonTariffied")) {

				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("CalendarWindow.errors.seasonTariffied")+ "</html>");

				Toolkit.getDefaultToolkit().beep();			
				return true;
			}

		}

		return false;

	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
